# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .network_acl_action import NetworkAclAction
from .network_acl_match import NetworkAclMatch
from .network_acl_rule_scope_enum import NetworkAclRuleScopeEnum


class NetworkAclRule(UniversalBaseModel):
    action: NetworkAclAction
    match: typing.Optional[NetworkAclMatch] = None
    not_match: typing.Optional[NetworkAclMatch] = None
    scope: NetworkAclRuleScopeEnum

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .user_authentication_method import UserAuthenticationMethod


class ListUserAuthenticationMethodsOffsetPaginatedResponseContent(UniversalBaseModel):
    start: typing.Optional[float] = pydantic.Field(default=None)
    """
    Index of the starting record. Derived from the page and per_page parameters.
    """

    limit: typing.Optional[float] = pydantic.Field(default=None)
    """
    Maximum amount of records to return.
    """

    total: typing.Optional[float] = pydantic.Field(default=None)
    """
    Total number of pageable records.
    """

    authenticators: typing.Optional[typing.List[UserAuthenticationMethod]] = pydantic.Field(default=None)
    """
    The paginated authentication methods. Returned in this structure when include_totals is true.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.get_phone_provider_protection_response_content import GetPhoneProviderProtectionResponseContent
from ...types.patch_phone_provider_protection_response_content import PatchPhoneProviderProtectionResponseContent
from ...types.phone_provider_protection_backoff_strategy_enum import PhoneProviderProtectionBackoffStrategyEnum
from .raw_client import AsyncRawPhoneProviderProtectionClient, RawPhoneProviderProtectionClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PhoneProviderProtectionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPhoneProviderProtectionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPhoneProviderProtectionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPhoneProviderProtectionClient
        """
        return self._raw_client

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetPhoneProviderProtectionResponseContent:
        """
        Get the phone provider protection configuration for a tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPhoneProviderProtectionResponseContent
            Phone provider protection configuration successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.attack_protection.phone_provider_protection.get()
        """
        _response = self._raw_client.get(request_options=request_options)
        return _response.data

    def patch(
        self,
        *,
        type: PhoneProviderProtectionBackoffStrategyEnum,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatchPhoneProviderProtectionResponseContent:
        """
        Update the phone provider protection configuration for a tenant.

        Parameters
        ----------
        type : PhoneProviderProtectionBackoffStrategyEnum

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatchPhoneProviderProtectionResponseContent
            Phone provider protection configuration successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.attack_protection.phone_provider_protection.patch(
            type="exponential",
        )
        """
        _response = self._raw_client.patch(type=type, request_options=request_options)
        return _response.data


class AsyncPhoneProviderProtectionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPhoneProviderProtectionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPhoneProviderProtectionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPhoneProviderProtectionClient
        """
        return self._raw_client

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetPhoneProviderProtectionResponseContent:
        """
        Get the phone provider protection configuration for a tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPhoneProviderProtectionResponseContent
            Phone provider protection configuration successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.attack_protection.phone_provider_protection.get()


        asyncio.run(main())
        """
        _response = await self._raw_client.get(request_options=request_options)
        return _response.data

    async def patch(
        self,
        *,
        type: PhoneProviderProtectionBackoffStrategyEnum,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatchPhoneProviderProtectionResponseContent:
        """
        Update the phone provider protection configuration for a tenant.

        Parameters
        ----------
        type : PhoneProviderProtectionBackoffStrategyEnum

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatchPhoneProviderProtectionResponseContent
            Phone provider protection configuration successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.attack_protection.phone_provider_protection.patch(
                type="exponential",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.patch(type=type, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

FlowsVaultConnectioSetupJwtAlgorithmEnum = typing.Union[
    typing.Literal[
        "HS256", "HS384", "HS512", "RS256", "RS384", "RS512", "ES256", "ES384", "ES512", "PS256", "PS384", "PS512"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.create_custom_domain_response_content import CreateCustomDomainResponseContent
from ..types.custom_domain_custom_client_ip_header import CustomDomainCustomClientIpHeader
from ..types.custom_domain_provisioning_type_enum import CustomDomainProvisioningTypeEnum
from ..types.custom_domain_tls_policy_enum import CustomDomainTlsPolicyEnum
from ..types.custom_domain_verification_method_enum import CustomDomainVerificationMethodEnum
from ..types.domain_metadata import DomainMetadata
from ..types.get_custom_domain_response_content import GetCustomDomainResponseContent
from ..types.get_default_domain_response_content import GetDefaultDomainResponseContent
from ..types.list_custom_domains_response_content import ListCustomDomainsResponseContent
from ..types.test_custom_domain_response_content import TestCustomDomainResponseContent
from ..types.update_custom_domain_response_content import UpdateCustomDomainResponseContent
from ..types.update_default_domain_response_content import UpdateDefaultDomainResponseContent
from ..types.verify_custom_domain_response_content import VerifyCustomDomainResponseContent
from .raw_client import AsyncRawCustomDomainsClient, RawCustomDomainsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomDomainsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomDomainsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomDomainsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomDomainsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        q: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        sort: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListCustomDomainsResponseContent:
        """
        Retrieve details on [custom domains](https://auth0.com/docs/custom-domains).

        Parameters
        ----------
        q : typing.Optional[str]
            Query in <a href ="https://lucene.apache.org/core/2_9_4/queryparsersyntax.html">Lucene query string syntax</a>.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        sort : typing.Optional[str]
            Field to sort by. Only <code>domain:1</code> (ascending order by domain) is supported at this time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListCustomDomainsResponseContent
            Custom domains successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.custom_domains.list(
            q="q",
            fields="fields",
            include_fields=True,
            sort="sort",
        )
        """
        _response = self._raw_client.list(
            q=q, fields=fields, include_fields=include_fields, sort=sort, request_options=request_options
        )
        return _response.data

    def create(
        self,
        *,
        domain: str,
        type: CustomDomainProvisioningTypeEnum,
        verification_method: typing.Optional[CustomDomainVerificationMethodEnum] = OMIT,
        tls_policy: typing.Optional[CustomDomainTlsPolicyEnum] = OMIT,
        custom_client_ip_header: typing.Optional[CustomDomainCustomClientIpHeader] = OMIT,
        domain_metadata: typing.Optional[DomainMetadata] = OMIT,
        relying_party_identifier: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCustomDomainResponseContent:
        """
        Create a new custom domain.

        Note: The custom domain will need to be verified before it will accept
        requests.

        Optional attributes that can be updated:

        - custom_client_ip_header
        - tls_policy

        TLS Policies:

        - recommended - for modern usage this includes TLS 1.2 only

        Parameters
        ----------
        domain : str
            Domain name.

        type : CustomDomainProvisioningTypeEnum

        verification_method : typing.Optional[CustomDomainVerificationMethodEnum]

        tls_policy : typing.Optional[CustomDomainTlsPolicyEnum]

        custom_client_ip_header : typing.Optional[CustomDomainCustomClientIpHeader]

        domain_metadata : typing.Optional[DomainMetadata]

        relying_party_identifier : typing.Optional[str]
            Relying Party ID (rpId) to be used for Passkeys on this custom domain. If not provided, the full domain will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomDomainResponseContent
            Custom domain successfully created (verification is pending).

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.custom_domains.create(
            domain="domain",
            type="auth0_managed_certs",
        )
        """
        _response = self._raw_client.create(
            domain=domain,
            type=type,
            verification_method=verification_method,
            tls_policy=tls_policy,
            custom_client_ip_header=custom_client_ip_header,
            domain_metadata=domain_metadata,
            relying_party_identifier=relying_party_identifier,
            request_options=request_options,
        )
        return _response.data

    def get_default(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetDefaultDomainResponseContent:
        """
        Retrieve the tenant's default domain.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDefaultDomainResponseContent
            Default domain successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.custom_domains.get_default()
        """
        _response = self._raw_client.get_default(request_options=request_options)
        return _response.data

    def set_default(
        self, *, domain: str, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateDefaultDomainResponseContent:
        """
        Set the default custom domain for the tenant.

        Parameters
        ----------
        domain : str
            The domain to set as the default custom domain. Must be a verified custom domain or the canonical domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateDefaultDomainResponseContent
            Default custom domain set successfully.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.custom_domains.set_default(
            domain="domain",
        )
        """
        _response = self._raw_client.set_default(domain=domain, request_options=request_options)
        return _response.data

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCustomDomainResponseContent:
        """
        Retrieve a custom domain configuration and status.

        Parameters
        ----------
        id : str
            ID of the custom domain to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomDomainResponseContent
            Custom domain successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.custom_domains.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a custom domain and stop serving requests for it.

        Parameters
        ----------
        id : str
            ID of the custom domain to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.custom_domains.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        tls_policy: typing.Optional[CustomDomainTlsPolicyEnum] = OMIT,
        custom_client_ip_header: typing.Optional[CustomDomainCustomClientIpHeader] = OMIT,
        domain_metadata: typing.Optional[DomainMetadata] = OMIT,
        relying_party_identifier: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateCustomDomainResponseContent:
        """
        Update a custom domain.

        These are the attributes that can be updated:

        - custom_client_ip_header
        - tls_policy

        **Updating CUSTOM_CLIENT_IP_HEADER for a custom domain**

        To update the `custom_client_ip_header` for a domain, the body to
        send should be:

        ```json
        { "custom_client_ip_header": "cf-connecting-ip" }
        ```

        **Updating TLS_POLICY for a custom domain**

        To update the `tls_policy` for a domain, the body to send should be:

        ```json
        { "tls_policy": "recommended" }
        ```

        TLS Policies:

        - recommended - for modern usage this includes TLS 1.2 only

        Some considerations:

        - The TLS ciphers and protocols available in each TLS policy follow industry recommendations, and may be updated occasionally.
        - The `compatible` TLS policy is no longer supported.

        Parameters
        ----------
        id : str
            The id of the custom domain to update

        tls_policy : typing.Optional[CustomDomainTlsPolicyEnum]
            recommended includes TLS 1.2

        custom_client_ip_header : typing.Optional[CustomDomainCustomClientIpHeader]

        domain_metadata : typing.Optional[DomainMetadata]

        relying_party_identifier : typing.Optional[str]
            Relying Party ID (rpId) to be used for Passkeys on this custom domain. Set to null to remove the rpId and fall back to using the full domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateCustomDomainResponseContent
            Custom domain updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.custom_domains.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id,
            tls_policy=tls_policy,
            custom_client_ip_header=custom_client_ip_header,
            domain_metadata=domain_metadata,
            relying_party_identifier=relying_party_identifier,
            request_options=request_options,
        )
        return _response.data

    def test(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TestCustomDomainResponseContent:
        """
        Run the test process on a custom domain.

        Parameters
        ----------
        id : str
            ID of the custom domain to test.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestCustomDomainResponseContent
            Custom domain test successfully completed.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.custom_domains.test(
            id="id",
        )
        """
        _response = self._raw_client.test(id, request_options=request_options)
        return _response.data

    def verify(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> VerifyCustomDomainResponseContent:
        """
        Run the verification process on a custom domain.

        Note: Check the `status` field to see its verification status. Once verification is complete, it may take up to 10 minutes before the custom domain can start accepting requests.

        For `self_managed_certs`, when the custom domain is verified for the first time, the response will also include the `cname_api_key` which you will need to configure your proxy. This key must be kept secret, and is used to validate the proxy requests.

        [Learn more](https://auth0.com/docs/custom-domains#step-2-verify-ownership) about verifying custom domains that use Auth0 Managed certificates.
        [Learn more](https://auth0.com/docs/custom-domains/self-managed-certificates#step-2-verify-ownership) about verifying custom domains that use Self Managed certificates.

        Parameters
        ----------
        id : str
            ID of the custom domain to verify.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VerifyCustomDomainResponseContent
            Custom domain successfully verified.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.custom_domains.verify(
            id="id",
        )
        """
        _response = self._raw_client.verify(id, request_options=request_options)
        return _response.data


class AsyncCustomDomainsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomDomainsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomDomainsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomDomainsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        q: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        sort: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListCustomDomainsResponseContent:
        """
        Retrieve details on [custom domains](https://auth0.com/docs/custom-domains).

        Parameters
        ----------
        q : typing.Optional[str]
            Query in <a href ="https://lucene.apache.org/core/2_9_4/queryparsersyntax.html">Lucene query string syntax</a>.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        sort : typing.Optional[str]
            Field to sort by. Only <code>domain:1</code> (ascending order by domain) is supported at this time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListCustomDomainsResponseContent
            Custom domains successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom_domains.list(
                q="q",
                fields="fields",
                include_fields=True,
                sort="sort",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            q=q, fields=fields, include_fields=include_fields, sort=sort, request_options=request_options
        )
        return _response.data

    async def create(
        self,
        *,
        domain: str,
        type: CustomDomainProvisioningTypeEnum,
        verification_method: typing.Optional[CustomDomainVerificationMethodEnum] = OMIT,
        tls_policy: typing.Optional[CustomDomainTlsPolicyEnum] = OMIT,
        custom_client_ip_header: typing.Optional[CustomDomainCustomClientIpHeader] = OMIT,
        domain_metadata: typing.Optional[DomainMetadata] = OMIT,
        relying_party_identifier: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCustomDomainResponseContent:
        """
        Create a new custom domain.

        Note: The custom domain will need to be verified before it will accept
        requests.

        Optional attributes that can be updated:

        - custom_client_ip_header
        - tls_policy

        TLS Policies:

        - recommended - for modern usage this includes TLS 1.2 only

        Parameters
        ----------
        domain : str
            Domain name.

        type : CustomDomainProvisioningTypeEnum

        verification_method : typing.Optional[CustomDomainVerificationMethodEnum]

        tls_policy : typing.Optional[CustomDomainTlsPolicyEnum]

        custom_client_ip_header : typing.Optional[CustomDomainCustomClientIpHeader]

        domain_metadata : typing.Optional[DomainMetadata]

        relying_party_identifier : typing.Optional[str]
            Relying Party ID (rpId) to be used for Passkeys on this custom domain. If not provided, the full domain will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomDomainResponseContent
            Custom domain successfully created (verification is pending).

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom_domains.create(
                domain="domain",
                type="auth0_managed_certs",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            domain=domain,
            type=type,
            verification_method=verification_method,
            tls_policy=tls_policy,
            custom_client_ip_header=custom_client_ip_header,
            domain_metadata=domain_metadata,
            relying_party_identifier=relying_party_identifier,
            request_options=request_options,
        )
        return _response.data

    async def get_default(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetDefaultDomainResponseContent:
        """
        Retrieve the tenant's default domain.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDefaultDomainResponseContent
            Default domain successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom_domains.get_default()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_default(request_options=request_options)
        return _response.data

    async def set_default(
        self, *, domain: str, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateDefaultDomainResponseContent:
        """
        Set the default custom domain for the tenant.

        Parameters
        ----------
        domain : str
            The domain to set as the default custom domain. Must be a verified custom domain or the canonical domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateDefaultDomainResponseContent
            Default custom domain set successfully.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom_domains.set_default(
                domain="domain",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set_default(domain=domain, request_options=request_options)
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCustomDomainResponseContent:
        """
        Retrieve a custom domain configuration and status.

        Parameters
        ----------
        id : str
            ID of the custom domain to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomDomainResponseContent
            Custom domain successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom_domains.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a custom domain and stop serving requests for it.

        Parameters
        ----------
        id : str
            ID of the custom domain to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom_domains.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        tls_policy: typing.Optional[CustomDomainTlsPolicyEnum] = OMIT,
        custom_client_ip_header: typing.Optional[CustomDomainCustomClientIpHeader] = OMIT,
        domain_metadata: typing.Optional[DomainMetadata] = OMIT,
        relying_party_identifier: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateCustomDomainResponseContent:
        """
        Update a custom domain.

        These are the attributes that can be updated:

        - custom_client_ip_header
        - tls_policy

        **Updating CUSTOM_CLIENT_IP_HEADER for a custom domain**

        To update the `custom_client_ip_header` for a domain, the body to
        send should be:

        ```json
        { "custom_client_ip_header": "cf-connecting-ip" }
        ```

        **Updating TLS_POLICY for a custom domain**

        To update the `tls_policy` for a domain, the body to send should be:

        ```json
        { "tls_policy": "recommended" }
        ```

        TLS Policies:

        - recommended - for modern usage this includes TLS 1.2 only

        Some considerations:

        - The TLS ciphers and protocols available in each TLS policy follow industry recommendations, and may be updated occasionally.
        - The `compatible` TLS policy is no longer supported.

        Parameters
        ----------
        id : str
            The id of the custom domain to update

        tls_policy : typing.Optional[CustomDomainTlsPolicyEnum]
            recommended includes TLS 1.2

        custom_client_ip_header : typing.Optional[CustomDomainCustomClientIpHeader]

        domain_metadata : typing.Optional[DomainMetadata]

        relying_party_identifier : typing.Optional[str]
            Relying Party ID (rpId) to be used for Passkeys on this custom domain. Set to null to remove the rpId and fall back to using the full domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateCustomDomainResponseContent
            Custom domain updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom_domains.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            tls_policy=tls_policy,
            custom_client_ip_header=custom_client_ip_header,
            domain_metadata=domain_metadata,
            relying_party_identifier=relying_party_identifier,
            request_options=request_options,
        )
        return _response.data

    async def test(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TestCustomDomainResponseContent:
        """
        Run the test process on a custom domain.

        Parameters
        ----------
        id : str
            ID of the custom domain to test.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestCustomDomainResponseContent
            Custom domain test successfully completed.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom_domains.test(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.test(id, request_options=request_options)
        return _response.data

    async def verify(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> VerifyCustomDomainResponseContent:
        """
        Run the verification process on a custom domain.

        Note: Check the `status` field to see its verification status. Once verification is complete, it may take up to 10 minutes before the custom domain can start accepting requests.

        For `self_managed_certs`, when the custom domain is verified for the first time, the response will also include the `cname_api_key` which you will need to configure your proxy. This key must be kept secret, and is used to validate the proxy requests.

        [Learn more](https://auth0.com/docs/custom-domains#step-2-verify-ownership) about verifying custom domains that use Auth0 Managed certificates.
        [Learn more](https://auth0.com/docs/custom-domains/self-managed-certificates#step-2-verify-ownership) about verifying custom domains that use Self Managed certificates.

        Parameters
        ----------
        id : str
            ID of the custom domain to verify.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VerifyCustomDomainResponseContent
            Custom domain successfully verified.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.custom_domains.verify(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.verify(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_options_client_id_github import ConnectionOptionsClientIdGithub
from .connection_options_client_secret_github import ConnectionOptionsClientSecretGithub
from .connection_options_common import ConnectionOptionsCommon
from .connection_options_freeform_scopes_github import ConnectionOptionsFreeformScopesGithub
from .connection_options_scope_github import ConnectionOptionsScopeGithub
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_upstream_params import ConnectionUpstreamParams


class ConnectionOptionsGitHub(ConnectionOptionsCommon):
    """
    Options for the 'github' connection
    """

    client_id: typing.Optional[ConnectionOptionsClientIdGithub] = None
    client_secret: typing.Optional[ConnectionOptionsClientSecretGithub] = None
    freeform_scopes: typing.Optional[ConnectionOptionsFreeformScopesGithub] = None
    scope: typing.Optional[ConnectionOptionsScopeGithub] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None
    admin_org: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the GitHub admin:org scope so Auth0 can fully manage organizations, teams, and memberships on behalf of the user.
    """

    admin_public_key: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the admin:public_key scope to allow creating, updating, and deleting the user's SSH public keys.
    """

    admin_repo_hook: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the admin:repo_hook scope so Auth0 can read, write, ping, and delete repository webhooks.
    """

    delete_repo: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the delete_repo scope so the user can remove repositories they administer while signing in through Auth0.
    """

    email: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the user:email scope so Auth0 pulls addresses from GitHub's /user/emails endpoint and populates the profile.
    """

    follow: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the user:follow scope to allow following or unfollowing GitHub users for the signed-in account.
    """

    gist: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the gist scope so the application can create or update gists on behalf of the user.
    """

    notifications: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the notifications scope to read GitHub inbox notifications; repo also implicitly grants this access.
    """

    profile: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Controls the GitHub read:user call that returns the user's basic profile (name, avatar, profile URL) and is on by default for successful logins.
    """

    public_repo: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the public_repo scope for read and write operations on public repositories, deployments, and statuses.
    """

    read_org: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the read:org scope so Auth0 can view organizations, teams, and membership lists without making changes.
    """

    read_public_key: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the read:public_key scope so Auth0 can list and inspect the user's SSH public keys.
    """

    read_repo_hook: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the read:repo_hook scope to read and ping repository webhooks.
    """

    read_user: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the read:user scope to load extended profile information, implicitly covering user:email and user:follow.
    """

    repo: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the repo scope for read and write access to both public and private repositories, deployments, and statuses.
    """

    repo_deployment: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the repo_deployment scope in order to read and write deployment statuses for repositories.
    """

    repo_status: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the repo:status scope to manage commit statuses on public and private repositories.
    """

    write_org: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the write:org scope so Auth0 can change whether organization memberships are publicized.
    """

    write_public_key: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the write:public_key scope to create or update SSH public keys for the user.
    """

    write_repo_hook: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Requests the write:repo_hook scope so Auth0 can read, create, update, and ping repository webhooks.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .network_acl_match_connecting_ipv_4_cidr import NetworkAclMatchConnectingIpv4Cidr
from .network_acl_match_connecting_ipv_6_cidr import NetworkAclMatchConnectingIpv6Cidr
from .network_acl_match_ipv_4_cidr import NetworkAclMatchIpv4Cidr
from .network_acl_match_ipv_6_cidr import NetworkAclMatchIpv6Cidr


class NetworkAclMatch(UniversalBaseModel):
    asns: typing.Optional[typing.List[int]] = None
    auth_0_managed: typing_extensions.Annotated[
        typing.Optional[typing.List[str]], FieldMetadata(alias="auth0_managed"), pydantic.Field(alias="auth0_managed")
    ] = None
    geo_country_codes: typing.Optional[typing.List[str]] = None
    geo_subdivision_codes: typing.Optional[typing.List[str]] = None
    ipv_4_cidrs: typing_extensions.Annotated[
        typing.Optional[typing.List[NetworkAclMatchIpv4Cidr]],
        FieldMetadata(alias="ipv4_cidrs"),
        pydantic.Field(alias="ipv4_cidrs"),
    ] = None
    ipv_6_cidrs: typing_extensions.Annotated[
        typing.Optional[typing.List[NetworkAclMatchIpv6Cidr]],
        FieldMetadata(alias="ipv6_cidrs"),
        pydantic.Field(alias="ipv6_cidrs"),
    ] = None
    ja_3_fingerprints: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="ja3_fingerprints"),
        pydantic.Field(alias="ja3_fingerprints"),
    ] = None
    ja_4_fingerprints: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="ja4_fingerprints"),
        pydantic.Field(alias="ja4_fingerprints"),
    ] = None
    user_agents: typing.Optional[typing.List[str]] = None
    hostnames: typing.Optional[typing.List[str]] = None
    connecting_ipv_4_cidrs: typing_extensions.Annotated[
        typing.Optional[typing.List[NetworkAclMatchConnectingIpv4Cidr]],
        FieldMetadata(alias="connecting_ipv4_cidrs"),
        pydantic.Field(alias="connecting_ipv4_cidrs"),
    ] = None
    connecting_ipv_6_cidrs: typing_extensions.Annotated[
        typing.Optional[typing.List[NetworkAclMatchConnectingIpv6Cidr]],
        FieldMetadata(alias="connecting_ipv6_cidrs"),
        pydantic.Field(alias="connecting_ipv6_cidrs"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_clearbit_find_company import FlowActionClearbitFindCompany
from .flow_action_clearbit_find_person import FlowActionClearbitFindPerson

FlowActionClearbit = typing.Union[FlowActionClearbitFindPerson, FlowActionClearbitFindCompany]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.assessors_type_enum import AssessorsTypeEnum
from .raw_client import AsyncRawRiskAssessmentsClient, RawRiskAssessmentsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RiskAssessmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRiskAssessmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawRiskAssessmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRiskAssessmentsClient
        """
        return self._raw_client

    def clear(
        self,
        id: str,
        *,
        connection: str,
        assessors: typing.Sequence[AssessorsTypeEnum],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Clear risk assessment assessors for a specific user

        Parameters
        ----------
        id : str
            ID of the user to clear assessors for.

        connection : str
            The name of the connection containing the user whose assessors should be cleared.

        assessors : typing.Sequence[AssessorsTypeEnum]
            List of assessors to clear.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.risk_assessments.clear(
            id="id",
            connection="connection",
            assessors=["new-device"],
        )
        """
        _response = self._raw_client.clear(
            id, connection=connection, assessors=assessors, request_options=request_options
        )
        return _response.data


class AsyncRiskAssessmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRiskAssessmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawRiskAssessmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRiskAssessmentsClient
        """
        return self._raw_client

    async def clear(
        self,
        id: str,
        *,
        connection: str,
        assessors: typing.Sequence[AssessorsTypeEnum],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Clear risk assessment assessors for a specific user

        Parameters
        ----------
        id : str
            ID of the user to clear assessors for.

        connection : str
            The name of the connection containing the user whose assessors should be cleared.

        assessors : typing.Sequence[AssessorsTypeEnum]
            List of assessors to clear.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.risk_assessments.clear(
                id="id",
                connection="connection",
                assessors=["new-device"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.clear(
            id, connection=connection, assessors=assessors, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connection_email_otp_authentication_method import ConnectionEmailOtpAuthenticationMethod
from .connection_passkey_authentication_method import ConnectionPasskeyAuthenticationMethod
from .connection_password_authentication_method import ConnectionPasswordAuthenticationMethod
from .connection_phone_otp_authentication_method import ConnectionPhoneOtpAuthenticationMethod


class ConnectionAuthenticationMethods(UniversalBaseModel):
    """
    Options for enabling authentication methods.
    """

    password: typing.Optional[ConnectionPasswordAuthenticationMethod] = None
    passkey: typing.Optional[ConnectionPasskeyAuthenticationMethod] = None
    email_otp: typing.Optional[ConnectionEmailOtpAuthenticationMethod] = None
    phone_otp: typing.Optional[ConnectionPhoneOtpAuthenticationMethod] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .app_metadata import AppMetadata
from .organization_invitation_invitee import OrganizationInvitationInvitee
from .organization_invitation_inviter import OrganizationInvitationInviter
from .user_metadata import UserMetadata


class GetOrganizationInvitationResponseContent(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the user invitation.
    """

    organization_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Organization identifier.
    """

    inviter: typing.Optional[OrganizationInvitationInviter] = None
    invitee: typing.Optional[OrganizationInvitationInvitee] = None
    invitation_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The invitation url to be send to the invitee.
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The ISO 8601 formatted timestamp representing the creation time of the invitation.
    """

    expires_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The ISO 8601 formatted timestamp representing the expiration time of the invitation.
    """

    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Auth0 client ID. Used to resolve the application's login initiation endpoint.
    """

    connection_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the connection to force invitee to authenticate with.
    """

    app_metadata: typing.Optional[AppMetadata] = None
    user_metadata: typing.Optional[UserMetadata] = None
    roles: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of roles IDs to associated with the user.
    """

    ticket_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the invitation ticket
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

ConnectionProfileBitbucket = bool
"""
`profile` is a boolean that controls whether basic profile information (name, picture, email) is requested from Bitbucket during authentication. When `true`, the connection requests access to the user's basic profile data.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

ClientAddonBox = typing.Dict[str, typing.Any]
"""
Box SSO indicator (no configuration settings needed for Box SSO).
"""



from .conftest import get_client, verify_request_count

from auth0.management import MdlPresentationProperties, MdlPresentationRequest, MdlPresentationRequestProperties


def test_verifiableCredentials_verification_templates_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "verifiable_credentials.verification.templates.list_.0"
    client = get_client(test_id)
    client.verifiable_credentials.verification.templates.list(
        from_="from",
        take=1,
    )
    verify_request_count(
        test_id, "GET", "/verifiable-credentials/verification/templates", {"from": "from", "take": "1"}, 1
    )


def test_verifiableCredentials_verification_templates_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "verifiable_credentials.verification.templates.create.0"
    client = get_client(test_id)
    client.verifiable_credentials.verification.templates.create(
        name="name",
        type="type",
        dialect="dialect",
        presentation=MdlPresentationRequest(
            org_iso_18013_5_1_m_dl=MdlPresentationRequestProperties(
                org_iso_18013_5_1=MdlPresentationProperties(),
            ),
        ),
        well_known_trusted_issuers="well_known_trusted_issuers",
    )
    verify_request_count(test_id, "POST", "/verifiable-credentials/verification/templates", None, 1)


def test_verifiableCredentials_verification_templates_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "verifiable_credentials.verification.templates.get.0"
    client = get_client(test_id)
    client.verifiable_credentials.verification.templates.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/verifiable-credentials/verification/templates/id", None, 1)


def test_verifiableCredentials_verification_templates_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "verifiable_credentials.verification.templates.delete.0"
    client = get_client(test_id)
    client.verifiable_credentials.verification.templates.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/verifiable-credentials/verification/templates/id", None, 1)


def test_verifiableCredentials_verification_templates_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "verifiable_credentials.verification.templates.update.0"
    client = get_client(test_id)
    client.verifiable_credentials.verification.templates.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/verifiable-credentials/verification/templates/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .log_stream_event_bridge_sink_region_enum import LogStreamEventBridgeSinkRegionEnum


class LogStreamEventBridgeSink(UniversalBaseModel):
    aws_account_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="awsAccountId"), pydantic.Field(alias="awsAccountId", description="AWS account ID")
    ]
    """
    AWS account ID
    """

    aws_region: typing_extensions.Annotated[
        LogStreamEventBridgeSinkRegionEnum, FieldMetadata(alias="awsRegion"), pydantic.Field(alias="awsRegion")
    ]
    aws_partner_event_source: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="awsPartnerEventSource"),
        pydantic.Field(alias="awsPartnerEventSource", description="AWS EventBridge partner event source"),
    ] = None
    """
    AWS EventBridge partner event source
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_bigquery_jwt import CreateFlowsVaultConnectionBigqueryJwt
from .create_flows_vault_connection_bigquery_uninitialized import CreateFlowsVaultConnectionBigqueryUninitialized

CreateFlowsVaultConnectionBigquery = typing.Union[
    CreateFlowsVaultConnectionBigqueryJwt, CreateFlowsVaultConnectionBigqueryUninitialized
]



# This file was auto-generated by Fern from our API Definition.

import typing

LogStreamFilterGroupNameEnum = typing.Union[
    typing.Literal[
        "auth.login.fail",
        "auth.login.notification",
        "auth.login.success",
        "auth.logout.fail",
        "auth.logout.success",
        "auth.signup.fail",
        "auth.signup.success",
        "auth.silent_auth.fail",
        "auth.silent_auth.success",
        "auth.token_exchange.fail",
        "auth.token_exchange.success",
        "management.fail",
        "management.success",
        "scim.event",
        "system.notification",
        "user.fail",
        "user.notification",
        "user.success",
        "actions",
        "other",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_connection_created_object_2_options_idpinitiated_client_protocol_enum import (
    EventStreamCloudEventConnectionCreatedObject2OptionsIdpinitiatedClientProtocolEnum,
)


class EventStreamCloudEventConnectionCreatedObject2OptionsIdpinitiated(UniversalBaseModel):
    """
    Configuration for IdP-Initiated SAML Single Sign-On. When enabled, allows users to initiate login directly from their SAML identity provider without first visiting Auth0. The IdP must include the connection parameter in the post-back URL (Assertion Consumer Service URL).
    """

    client_authorizequery: typing.Optional[str] = pydantic.Field(default=None)
    """
    The query string sent to the default application
    """

    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The client ID to use for IdP-initiated login requests.
    """

    client_protocol: typing.Optional[
        EventStreamCloudEventConnectionCreatedObject2OptionsIdpinitiatedClientProtocolEnum
    ] = None
    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When true, enables IdP-initiated login support for this SAML connection. Allows users to log in directly from the identity provider without first visiting Auth0.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_group_role_assigned_object_group_1_type_enum import (
    EventStreamCloudEventGroupRoleAssignedObjectGroup1TypeEnum,
)


class EventStreamCloudEventGroupRoleAssignedObjectGroup1(UniversalBaseModel):
    """
    Reference to an organization group
    """

    id: str = pydantic.Field()
    """
    The unique identifier for the group.
    """

    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The external identifier for the group.
    """

    type: EventStreamCloudEventGroupRoleAssignedObjectGroup1TypeEnum
    organization_id: str = pydantic.Field()
    """
    The organization ID associated with the group.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

AuthenticationMethodTypeEnum = typing.Union[
    typing.Literal[
        "recovery-code",
        "totp",
        "push",
        "phone",
        "email",
        "email-verification",
        "webauthn-roaming",
        "webauthn-platform",
        "guardian",
        "passkey",
        "password",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.pagination import AsyncPager, SyncPager
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.connection_authentication_purpose import ConnectionAuthenticationPurpose
from ..types.connection_connected_accounts_purpose import ConnectionConnectedAccountsPurpose
from ..types.connection_for_list import ConnectionForList
from ..types.connection_identity_provider_enum import ConnectionIdentityProviderEnum
from ..types.connection_properties_options import ConnectionPropertiesOptions
from ..types.connection_strategy_enum import ConnectionStrategyEnum
from ..types.connections_metadata import ConnectionsMetadata
from ..types.create_connection_response_content import CreateConnectionResponseContent
from ..types.create_cross_app_access_resource_app import CreateCrossAppAccessResourceApp
from ..types.cross_app_access_requesting_app import CrossAppAccessRequestingApp
from ..types.get_connection_response_content import GetConnectionResponseContent
from ..types.list_connections_checkpoint_paginated_response_content import (
    ListConnectionsCheckpointPaginatedResponseContent,
)
from ..types.update_connection_options import UpdateConnectionOptions
from ..types.update_connection_response_content import UpdateConnectionResponseContent
from ..types.update_cross_app_access_resource_app import UpdateCrossAppAccessResourceApp
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawConnectionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        strategy: typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]] = None,
        name: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ConnectionForList, ListConnectionsCheckpointPaginatedResponseContent]:
        """
        Retrieves detailed list of all [connections](https://auth0.com/docs/authenticate/identity-providers) that match the specified strategy. If no strategy is provided, all connections within your tenant are retrieved. This action can accept a list of fields to include or exclude from the resulting list of connections.

        This endpoint supports two types of pagination:

        - Offset pagination
        - Checkpoint pagination

        Checkpoint pagination must be used if you need to retrieve more than 1000 connections.

        **Checkpoint Pagination**

        To search by checkpoint, use the following parameters:

        - `from`: Optional id from which to start selection.
        - `take`: The total amount of entries to retrieve when using the from parameter. Defaults to 50.

        **Note**: The first time you call this endpoint using checkpoint pagination, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no pages are remaining.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        strategy : typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]]
            Provide strategies to only retrieve connections with such strategies

        name : typing.Optional[str]
            Provide the name of the connection to retrieve

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            <code>true</code> if the fields specified are to be included in the result, <code>false</code> otherwise (defaults to <code>true</code>)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ConnectionForList, ListConnectionsCheckpointPaginatedResponseContent]
            The connections were retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "connections",
            method="GET",
            params={
                "from": from_,
                "take": take,
                "strategy": strategy,
                "name": name,
                "fields": fields,
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListConnectionsCheckpointPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListConnectionsCheckpointPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.connections
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    from_=_parsed_next,
                    take=take,
                    strategy=strategy,
                    name=name,
                    fields=fields,
                    include_fields=include_fields,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        strategy: ConnectionIdentityProviderEnum,
        display_name: typing.Optional[str] = OMIT,
        options: typing.Optional[ConnectionPropertiesOptions] = OMIT,
        enabled_clients: typing.Optional[typing.Sequence[str]] = OMIT,
        is_domain_connection: typing.Optional[bool] = OMIT,
        show_as_button: typing.Optional[bool] = OMIT,
        realms: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[ConnectionsMetadata] = OMIT,
        authentication: typing.Optional[ConnectionAuthenticationPurpose] = OMIT,
        connected_accounts: typing.Optional[ConnectionConnectedAccountsPurpose] = OMIT,
        cross_app_access_requesting_app: typing.Optional[CrossAppAccessRequestingApp] = OMIT,
        cross_app_access_resource_app: typing.Optional[CreateCrossAppAccessResourceApp] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateConnectionResponseContent]:
        """
        Creates a new connection according to the JSON object received in `body`.

        **Note:** If a connection with the same name was recently deleted and had a large number of associated users, the deletion may still be processing. Creating a new connection with that name before the deletion completes may fail or produce unexpected results.

        Parameters
        ----------
        name : str
            The name of the connection. Must start and end with an alphanumeric character and can only contain alphanumeric characters and '-'. Max length 128

        strategy : ConnectionIdentityProviderEnum

        display_name : typing.Optional[str]
            Connection name used in the new universal login experience

        options : typing.Optional[ConnectionPropertiesOptions]

        enabled_clients : typing.Optional[typing.Sequence[str]]
            Use of this property is NOT RECOMMENDED. Use the PATCH /v2/connections/{id}/clients endpoint to enable the connection for a set of clients.

        is_domain_connection : typing.Optional[bool]
            <code>true</code> promotes to a domain-level connection so that third-party applications can use it. <code>false</code> does not promote the connection, so only first-party applications with the connection enabled can use it. (Defaults to <code>false</code>.)

        show_as_button : typing.Optional[bool]
            Enables showing a button for the connection in the login page (new experience only). If false, it will be usable only by HRD. (Defaults to <code>false</code>.)

        realms : typing.Optional[typing.Sequence[str]]
            Defines the realms for which the connection will be used (ie: email domains). If the array is empty or the property is not specified, the connection name will be added as realm.

        metadata : typing.Optional[ConnectionsMetadata]

        authentication : typing.Optional[ConnectionAuthenticationPurpose]

        connected_accounts : typing.Optional[ConnectionConnectedAccountsPurpose]

        cross_app_access_requesting_app : typing.Optional[CrossAppAccessRequestingApp]

        cross_app_access_resource_app : typing.Optional[CreateCrossAppAccessResourceApp]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateConnectionResponseContent]
            The connection was created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "connections",
            method="POST",
            json={
                "name": name,
                "display_name": display_name,
                "strategy": strategy,
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=ConnectionPropertiesOptions, direction="write"
                ),
                "enabled_clients": enabled_clients,
                "is_domain_connection": is_domain_connection,
                "show_as_button": show_as_button,
                "realms": realms,
                "metadata": metadata,
                "authentication": convert_and_respect_annotation_metadata(
                    object_=authentication, annotation=ConnectionAuthenticationPurpose, direction="write"
                ),
                "connected_accounts": convert_and_respect_annotation_metadata(
                    object_=connected_accounts, annotation=ConnectionConnectedAccountsPurpose, direction="write"
                ),
                "cross_app_access_requesting_app": convert_and_respect_annotation_metadata(
                    object_=cross_app_access_requesting_app, annotation=CrossAppAccessRequestingApp, direction="write"
                ),
                "cross_app_access_resource_app": convert_and_respect_annotation_metadata(
                    object_=cross_app_access_resource_app, annotation=CreateCrossAppAccessResourceApp, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateConnectionResponseContent,
                    parse_obj_as(
                        type_=CreateConnectionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self,
        id: str,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetConnectionResponseContent]:
        """
        Retrieve details for a specified [connection](https://auth0.com/docs/authenticate/identity-providers) along with options that can be used for identity provider configuration.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            <code>true</code> if the fields specified are to be included in the result, <code>false</code> otherwise (defaults to <code>true</code>)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetConnectionResponseContent]
            The connection was retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"connections/{encode_path_param(id)}",
            method="GET",
            params={
                "fields": fields,
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConnectionResponseContent,
                    parse_obj_as(
                        type_=GetConnectionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Removes a specific [connection](https://auth0.com/docs/authenticate/identity-providers) from your tenant. This action cannot be undone. Once removed, users can no longer use this connection to authenticate.

        **Note:** If your connection has a large amount of users associated with it, please be aware that this operation can be long running after the response is returned and may impact concurrent [create connection](https://auth0.com/docs/api/management/v2/connections/post-connections) requests, if they use an identical connection name.

        Parameters
        ----------
        id : str
            The id of the connection to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"connections/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        display_name: typing.Optional[str] = OMIT,
        options: typing.Optional[UpdateConnectionOptions] = OMIT,
        enabled_clients: typing.Optional[typing.Sequence[str]] = OMIT,
        is_domain_connection: typing.Optional[bool] = OMIT,
        show_as_button: typing.Optional[bool] = OMIT,
        realms: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[ConnectionsMetadata] = OMIT,
        authentication: typing.Optional[ConnectionAuthenticationPurpose] = OMIT,
        connected_accounts: typing.Optional[ConnectionConnectedAccountsPurpose] = OMIT,
        cross_app_access_requesting_app: typing.Optional[CrossAppAccessRequestingApp] = OMIT,
        cross_app_access_resource_app: typing.Optional[UpdateCrossAppAccessResourceApp] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateConnectionResponseContent]:
        """
        Update details for a specific [connection](https://auth0.com/docs/authenticate/identity-providers), including option properties for identity provider configuration.

        **Note**: If you use the `options` parameter, the entire `options` object is overridden. To avoid partial data or other issues, ensure all parameters are present when using this option.

        Parameters
        ----------
        id : str
            The id of the connection to update

        display_name : typing.Optional[str]
            The connection name used in the new universal login experience. If display_name is not included in the request, the field will be overwritten with the name value.

        options : typing.Optional[UpdateConnectionOptions]

        enabled_clients : typing.Optional[typing.Sequence[str]]
            DEPRECATED property. Use the PATCH /v2/connections/{id}/clients endpoint to enable or disable the connection for any clients.

        is_domain_connection : typing.Optional[bool]
            <code>true</code> promotes to a domain-level connection so that third-party applications can use it. <code>false</code> does not promote the connection, so only first-party applications with the connection enabled can use it. (Defaults to <code>false</code>.)

        show_as_button : typing.Optional[bool]
            Enables showing a button for the connection in the login page (new experience only). If false, it will be usable only by HRD. (Defaults to <code>false</code>.)

        realms : typing.Optional[typing.Sequence[str]]
            Defines the realms for which the connection will be used (ie: email domains). If the array is empty or the property is not specified, the connection name will be added as realm.

        metadata : typing.Optional[ConnectionsMetadata]

        authentication : typing.Optional[ConnectionAuthenticationPurpose]

        connected_accounts : typing.Optional[ConnectionConnectedAccountsPurpose]

        cross_app_access_requesting_app : typing.Optional[CrossAppAccessRequestingApp]

        cross_app_access_resource_app : typing.Optional[UpdateCrossAppAccessResourceApp]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateConnectionResponseContent]
            The connection was updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"connections/{encode_path_param(id)}",
            method="PATCH",
            json={
                "display_name": display_name,
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=typing.Optional[UpdateConnectionOptions], direction="write"
                ),
                "enabled_clients": enabled_clients,
                "is_domain_connection": is_domain_connection,
                "show_as_button": show_as_button,
                "realms": realms,
                "metadata": metadata,
                "authentication": convert_and_respect_annotation_metadata(
                    object_=authentication, annotation=ConnectionAuthenticationPurpose, direction="write"
                ),
                "connected_accounts": convert_and_respect_annotation_metadata(
                    object_=connected_accounts, annotation=ConnectionConnectedAccountsPurpose, direction="write"
                ),
                "cross_app_access_requesting_app": convert_and_respect_annotation_metadata(
                    object_=cross_app_access_requesting_app, annotation=CrossAppAccessRequestingApp, direction="write"
                ),
                "cross_app_access_resource_app": convert_and_respect_annotation_metadata(
                    object_=cross_app_access_resource_app,
                    annotation=typing.Optional[UpdateCrossAppAccessResourceApp],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateConnectionResponseContent,
                    parse_obj_as(
                        type_=UpdateConnectionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def check_status(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Retrieves the status of an ad/ldap connection referenced by its `ID`. `200 OK` http status code response is returned  when the connection is online, otherwise a `404` status code is returned along with an error message

        Parameters
        ----------
        id : str
            ID of the connection to check

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"connections/{encode_path_param(id)}/status",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawConnectionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        strategy: typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]] = None,
        name: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ConnectionForList, ListConnectionsCheckpointPaginatedResponseContent]:
        """
        Retrieves detailed list of all [connections](https://auth0.com/docs/authenticate/identity-providers) that match the specified strategy. If no strategy is provided, all connections within your tenant are retrieved. This action can accept a list of fields to include or exclude from the resulting list of connections.

        This endpoint supports two types of pagination:

        - Offset pagination
        - Checkpoint pagination

        Checkpoint pagination must be used if you need to retrieve more than 1000 connections.

        **Checkpoint Pagination**

        To search by checkpoint, use the following parameters:

        - `from`: Optional id from which to start selection.
        - `take`: The total amount of entries to retrieve when using the from parameter. Defaults to 50.

        **Note**: The first time you call this endpoint using checkpoint pagination, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no pages are remaining.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        strategy : typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]]
            Provide strategies to only retrieve connections with such strategies

        name : typing.Optional[str]
            Provide the name of the connection to retrieve

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            <code>true</code> if the fields specified are to be included in the result, <code>false</code> otherwise (defaults to <code>true</code>)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ConnectionForList, ListConnectionsCheckpointPaginatedResponseContent]
            The connections were retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "connections",
            method="GET",
            params={
                "from": from_,
                "take": take,
                "strategy": strategy,
                "name": name,
                "fields": fields,
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListConnectionsCheckpointPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListConnectionsCheckpointPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.connections
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""

                async def _get_next():
                    return await self.list(
                        from_=_parsed_next,
                        take=take,
                        strategy=strategy,
                        name=name,
                        fields=fields,
                        include_fields=include_fields,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        strategy: ConnectionIdentityProviderEnum,
        display_name: typing.Optional[str] = OMIT,
        options: typing.Optional[ConnectionPropertiesOptions] = OMIT,
        enabled_clients: typing.Optional[typing.Sequence[str]] = OMIT,
        is_domain_connection: typing.Optional[bool] = OMIT,
        show_as_button: typing.Optional[bool] = OMIT,
        realms: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[ConnectionsMetadata] = OMIT,
        authentication: typing.Optional[ConnectionAuthenticationPurpose] = OMIT,
        connected_accounts: typing.Optional[ConnectionConnectedAccountsPurpose] = OMIT,
        cross_app_access_requesting_app: typing.Optional[CrossAppAccessRequestingApp] = OMIT,
        cross_app_access_resource_app: typing.Optional[CreateCrossAppAccessResourceApp] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateConnectionResponseContent]:
        """
        Creates a new connection according to the JSON object received in `body`.

        **Note:** If a connection with the same name was recently deleted and had a large number of associated users, the deletion may still be processing. Creating a new connection with that name before the deletion completes may fail or produce unexpected results.

        Parameters
        ----------
        name : str
            The name of the connection. Must start and end with an alphanumeric character and can only contain alphanumeric characters and '-'. Max length 128

        strategy : ConnectionIdentityProviderEnum

        display_name : typing.Optional[str]
            Connection name used in the new universal login experience

        options : typing.Optional[ConnectionPropertiesOptions]

        enabled_clients : typing.Optional[typing.Sequence[str]]
            Use of this property is NOT RECOMMENDED. Use the PATCH /v2/connections/{id}/clients endpoint to enable the connection for a set of clients.

        is_domain_connection : typing.Optional[bool]
            <code>true</code> promotes to a domain-level connection so that third-party applications can use it. <code>false</code> does not promote the connection, so only first-party applications with the connection enabled can use it. (Defaults to <code>false</code>.)

        show_as_button : typing.Optional[bool]
            Enables showing a button for the connection in the login page (new experience only). If false, it will be usable only by HRD. (Defaults to <code>false</code>.)

        realms : typing.Optional[typing.Sequence[str]]
            Defines the realms for which the connection will be used (ie: email domains). If the array is empty or the property is not specified, the connection name will be added as realm.

        metadata : typing.Optional[ConnectionsMetadata]

        authentication : typing.Optional[ConnectionAuthenticationPurpose]

        connected_accounts : typing.Optional[ConnectionConnectedAccountsPurpose]

        cross_app_access_requesting_app : typing.Optional[CrossAppAccessRequestingApp]

        cross_app_access_resource_app : typing.Optional[CreateCrossAppAccessResourceApp]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateConnectionResponseContent]
            The connection was created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "connections",
            method="POST",
            json={
                "name": name,
                "display_name": display_name,
                "strategy": strategy,
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=ConnectionPropertiesOptions, direction="write"
                ),
                "enabled_clients": enabled_clients,
                "is_domain_connection": is_domain_connection,
                "show_as_button": show_as_button,
                "realms": realms,
                "metadata": metadata,
                "authentication": convert_and_respect_annotation_metadata(
                    object_=authentication, annotation=ConnectionAuthenticationPurpose, direction="write"
                ),
                "connected_accounts": convert_and_respect_annotation_metadata(
                    object_=connected_accounts, annotation=ConnectionConnectedAccountsPurpose, direction="write"
                ),
                "cross_app_access_requesting_app": convert_and_respect_annotation_metadata(
                    object_=cross_app_access_requesting_app, annotation=CrossAppAccessRequestingApp, direction="write"
                ),
                "cross_app_access_resource_app": convert_and_respect_annotation_metadata(
                    object_=cross_app_access_resource_app, annotation=CreateCrossAppAccessResourceApp, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateConnectionResponseContent,
                    parse_obj_as(
                        type_=CreateConnectionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self,
        id: str,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetConnectionResponseContent]:
        """
        Retrieve details for a specified [connection](https://auth0.com/docs/authenticate/identity-providers) along with options that can be used for identity provider configuration.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            <code>true</code> if the fields specified are to be included in the result, <code>false</code> otherwise (defaults to <code>true</code>)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetConnectionResponseContent]
            The connection was retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"connections/{encode_path_param(id)}",
            method="GET",
            params={
                "fields": fields,
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConnectionResponseContent,
                    parse_obj_as(
                        type_=GetConnectionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Removes a specific [connection](https://auth0.com/docs/authenticate/identity-providers) from your tenant. This action cannot be undone. Once removed, users can no longer use this connection to authenticate.

        **Note:** If your connection has a large amount of users associated with it, please be aware that this operation can be long running after the response is returned and may impact concurrent [create connection](https://auth0.com/docs/api/management/v2/connections/post-connections) requests, if they use an identical connection name.

        Parameters
        ----------
        id : str
            The id of the connection to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"connections/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        display_name: typing.Optional[str] = OMIT,
        options: typing.Optional[UpdateConnectionOptions] = OMIT,
        enabled_clients: typing.Optional[typing.Sequence[str]] = OMIT,
        is_domain_connection: typing.Optional[bool] = OMIT,
        show_as_button: typing.Optional[bool] = OMIT,
        realms: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[ConnectionsMetadata] = OMIT,
        authentication: typing.Optional[ConnectionAuthenticationPurpose] = OMIT,
        connected_accounts: typing.Optional[ConnectionConnectedAccountsPurpose] = OMIT,
        cross_app_access_requesting_app: typing.Optional[CrossAppAccessRequestingApp] = OMIT,
        cross_app_access_resource_app: typing.Optional[UpdateCrossAppAccessResourceApp] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateConnectionResponseContent]:
        """
        Update details for a specific [connection](https://auth0.com/docs/authenticate/identity-providers), including option properties for identity provider configuration.

        **Note**: If you use the `options` parameter, the entire `options` object is overridden. To avoid partial data or other issues, ensure all parameters are present when using this option.

        Parameters
        ----------
        id : str
            The id of the connection to update

        display_name : typing.Optional[str]
            The connection name used in the new universal login experience. If display_name is not included in the request, the field will be overwritten with the name value.

        options : typing.Optional[UpdateConnectionOptions]

        enabled_clients : typing.Optional[typing.Sequence[str]]
            DEPRECATED property. Use the PATCH /v2/connections/{id}/clients endpoint to enable or disable the connection for any clients.

        is_domain_connection : typing.Optional[bool]
            <code>true</code> promotes to a domain-level connection so that third-party applications can use it. <code>false</code> does not promote the connection, so only first-party applications with the connection enabled can use it. (Defaults to <code>false</code>.)

        show_as_button : typing.Optional[bool]
            Enables showing a button for the connection in the login page (new experience only). If false, it will be usable only by HRD. (Defaults to <code>false</code>.)

        realms : typing.Optional[typing.Sequence[str]]
            Defines the realms for which the connection will be used (ie: email domains). If the array is empty or the property is not specified, the connection name will be added as realm.

        metadata : typing.Optional[ConnectionsMetadata]

        authentication : typing.Optional[ConnectionAuthenticationPurpose]

        connected_accounts : typing.Optional[ConnectionConnectedAccountsPurpose]

        cross_app_access_requesting_app : typing.Optional[CrossAppAccessRequestingApp]

        cross_app_access_resource_app : typing.Optional[UpdateCrossAppAccessResourceApp]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateConnectionResponseContent]
            The connection was updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"connections/{encode_path_param(id)}",
            method="PATCH",
            json={
                "display_name": display_name,
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=typing.Optional[UpdateConnectionOptions], direction="write"
                ),
                "enabled_clients": enabled_clients,
                "is_domain_connection": is_domain_connection,
                "show_as_button": show_as_button,
                "realms": realms,
                "metadata": metadata,
                "authentication": convert_and_respect_annotation_metadata(
                    object_=authentication, annotation=ConnectionAuthenticationPurpose, direction="write"
                ),
                "connected_accounts": convert_and_respect_annotation_metadata(
                    object_=connected_accounts, annotation=ConnectionConnectedAccountsPurpose, direction="write"
                ),
                "cross_app_access_requesting_app": convert_and_respect_annotation_metadata(
                    object_=cross_app_access_requesting_app, annotation=CrossAppAccessRequestingApp, direction="write"
                ),
                "cross_app_access_resource_app": convert_and_respect_annotation_metadata(
                    object_=cross_app_access_resource_app,
                    annotation=typing.Optional[UpdateCrossAppAccessResourceApp],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateConnectionResponseContent,
                    parse_obj_as(
                        type_=UpdateConnectionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def check_status(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Retrieves the status of an ad/ldap connection referenced by its `ID`. `200 OK` http status code response is returned  when the connection is online, otherwise a `404` status code is returned along with an error message

        Parameters
        ----------
        id : str
            ID of the connection to check

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"connections/{encode_path_param(id)}/status",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



import unittest
from unittest import mock

from auth0.authentication.passwordless import Passwordless


class TestPasswordless(unittest.TestCase):
    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_send_email(self, mock_post):
        p = Passwordless("my.domain.com", "cid")

        p.email(email="a@b.com", send="snd")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/passwordless/start")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "email": "a@b.com",
                "send": "snd",
                "connection": "email",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_send_email_with_auth_params(self, mock_post):
        p = Passwordless("my.domain.com", "cid")

        p.email(email="a@b.com", send="snd", auth_params={"a": "b"})

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/passwordless/start")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "email": "a@b.com",
                "send": "snd",
                "authParams": {"a": "b"},
                "connection": "email",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_send_email_with_client_secret(self, mock_post):
        p = Passwordless("my.domain.com", "cid", client_secret="csecret")

        p.email(email="a@b.com", send="snd")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/passwordless/start")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "csecret",
                "email": "a@b.com",
                "send": "snd",
                "connection": "email",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_send_sms(self, mock_post):
        p = Passwordless("my.domain.com", "cid")

        p.sms(phone_number="123456")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/passwordless/start")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "phone_number": "123456",
                "connection": "sms",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_send_sms_with_client_secret(self, mock_post):
        p = Passwordless("my.domain.com", "cid", client_secret="csecret")

        p.sms(phone_number="123456")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/passwordless/start")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "csecret",
                "phone_number": "123456",
                "connection": "sms",
            },
        )



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.action_version import ActionVersion
from ...types.deploy_action_version_request_content import DeployActionVersionRequestContent
from ...types.deploy_action_version_response_content import DeployActionVersionResponseContent
from ...types.get_action_version_response_content import GetActionVersionResponseContent
from ...types.list_action_versions_paginated_response_content import ListActionVersionsPaginatedResponseContent
from .raw_client import AsyncRawVersionsClient, RawVersionsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class VersionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawVersionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawVersionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawVersionsClient
        """
        return self._raw_client

    def list(
        self,
        action_id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ActionVersion, ListActionVersionsPaginatedResponseContent]:
        """
        Retrieve all of an action's versions. An action version is created whenever an action is deployed. An action version is immutable, once created.

        Parameters
        ----------
        action_id : str
            The ID of the action.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            This field specify the maximum number of results to be returned by the server. 20 by default

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ActionVersion, ListActionVersionsPaginatedResponseContent]
            The action versions were retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.actions.versions.list(
            action_id="actionId",
            page=1,
            per_page=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(action_id, page=page, per_page=per_page, request_options=request_options)

    def get(
        self, action_id: str, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetActionVersionResponseContent:
        """
        Retrieve a specific version of an action. An action version is created whenever an action is deployed. An action version is immutable, once created.

        Parameters
        ----------
        action_id : str
            The ID of the action.

        id : str
            The ID of the action version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetActionVersionResponseContent
            The action version was retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.versions.get(
            action_id="actionId",
            id="id",
        )
        """
        _response = self._raw_client.get(action_id, id, request_options=request_options)
        return _response.data

    def deploy(
        self,
        action_id: str,
        id: str,
        *,
        request: typing.Optional[DeployActionVersionRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DeployActionVersionResponseContent:
        """
        Performs the equivalent of a roll-back of an action to an earlier, specified version. Creates a new, deployed action version that is identical to the specified version. If this action is currently bound to a trigger, the system will begin executing the newly-created version immediately.

        Parameters
        ----------
        action_id : str
            The ID of an action.

        id : str
            The ID of an action version.

        request : typing.Optional[DeployActionVersionRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeployActionVersionResponseContent
            Request to create action version was accepted.

        Examples
        --------
        from auth0 import Auth0, DeployActionVersionRequestContent

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.versions.deploy(
            action_id="actionId",
            id="id",
            request=DeployActionVersionRequestContent(),
        )
        """
        _response = self._raw_client.deploy(action_id, id, request=request, request_options=request_options)
        return _response.data


class AsyncVersionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawVersionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawVersionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawVersionsClient
        """
        return self._raw_client

    async def list(
        self,
        action_id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ActionVersion, ListActionVersionsPaginatedResponseContent]:
        """
        Retrieve all of an action's versions. An action version is created whenever an action is deployed. An action version is immutable, once created.

        Parameters
        ----------
        action_id : str
            The ID of the action.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            This field specify the maximum number of results to be returned by the server. 20 by default

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ActionVersion, ListActionVersionsPaginatedResponseContent]
            The action versions were retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.actions.versions.list(
                action_id="actionId",
                page=1,
                per_page=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(action_id, page=page, per_page=per_page, request_options=request_options)

    async def get(
        self, action_id: str, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetActionVersionResponseContent:
        """
        Retrieve a specific version of an action. An action version is created whenever an action is deployed. An action version is immutable, once created.

        Parameters
        ----------
        action_id : str
            The ID of the action.

        id : str
            The ID of the action version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetActionVersionResponseContent
            The action version was retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.versions.get(
                action_id="actionId",
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(action_id, id, request_options=request_options)
        return _response.data

    async def deploy(
        self,
        action_id: str,
        id: str,
        *,
        request: typing.Optional[DeployActionVersionRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DeployActionVersionResponseContent:
        """
        Performs the equivalent of a roll-back of an action to an earlier, specified version. Creates a new, deployed action version that is identical to the specified version. If this action is currently bound to a trigger, the system will begin executing the newly-created version immediately.

        Parameters
        ----------
        action_id : str
            The ID of an action.

        id : str
            The ID of an action version.

        request : typing.Optional[DeployActionVersionRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeployActionVersionResponseContent
            Request to create action version was accepted.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, DeployActionVersionRequestContent

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.versions.deploy(
                action_id="actionId",
                id="id",
                request=DeployActionVersionRequestContent(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.deploy(action_id, id, request=request, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

ConnectionBruteForceProtection = bool
"""
Enables Auth0's brute force protection to prevent credential stuffing attacks. When enabled, blocks suspicious login attempts from specific IP addresses after repeated failures.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .network_acl_action_allow_enum import NetworkAclActionAllowEnum
from .network_acl_action_block_enum import NetworkAclActionBlockEnum
from .network_acl_action_log_enum import NetworkAclActionLogEnum
from .network_acl_action_redirect_enum import NetworkAclActionRedirectEnum


class NetworkAclAction(UniversalBaseModel):
    block: typing.Optional[NetworkAclActionBlockEnum] = None
    allow: typing.Optional[NetworkAclActionAllowEnum] = None
    log: typing.Optional[NetworkAclActionLogEnum] = None
    redirect: typing.Optional[NetworkAclActionRedirectEnum] = None
    redirect_uri: typing.Optional[str] = pydantic.Field(default=None)
    """
    The URI to which the match or not_match requests will be routed
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionClaimTypesSupported = typing.List[str]
"""
JSON array containing a list of the Claim Types that the OpenID Provider supports. These Claim Types are described in Section 5.6 of OpenID Connect Core 1.0 [OpenID.Core]. If omitted, the implementation supports only normal Claims.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class EmailProviderCredentialsSchemaClientId(UniversalBaseModel):
    tenant_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="tenantId"),
        pydantic.Field(alias="tenantId", description="Microsoft 365 Tenant ID."),
    ] = None
    """
    Microsoft 365 Tenant ID.
    """

    client_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="clientId"),
        pydantic.Field(alias="clientId", description="Microsoft 365 Client ID."),
    ] = None
    """
    Microsoft 365 Client ID.
    """

    client_secret: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="clientSecret"),
        pydantic.Field(alias="clientSecret", description="Microsoft 365 Client Secret."),
    ] = None
    """
    Microsoft 365 Client Secret.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .connection_gateway_authentication_audience_sms import ConnectionGatewayAuthenticationAudienceSms
from .connection_gateway_authentication_method_sms import ConnectionGatewayAuthenticationMethodSms
from .connection_gateway_authentication_subject_sms import ConnectionGatewayAuthenticationSubjectSms


class ConnectionGatewayAuthenticationSms(UniversalBaseModel):
    """
    Optional token-based authentication configuration for the SMS gateway
    """

    audience: ConnectionGatewayAuthenticationAudienceSms
    method: ConnectionGatewayAuthenticationMethodSms
    secret: str = pydantic.Field()
    """
    The secret used to sign the JSON Web Token sent to the SMS gateway
    """

    secret_base_64_encoded: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="secret_base64_encoded"),
        pydantic.Field(alias="secret_base64_encoded", description="Set to true if the secret is base64-url-encoded"),
    ] = None
    """
    Set to true if the secret is base64-url-encoded
    """

    subject: typing.Optional[ConnectionGatewayAuthenticationSubjectSms] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_jobs_usersImports_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "jobs.users_imports.create.0"
    client = get_client(test_id)
    client.jobs.users_imports.create(
        users="example_users",
        connection_id="connection_id",
    )
    verify_request_count(test_id, "POST", "/jobs/users-imports", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from .form_field_boolean import FormFieldBoolean
from .form_field_cards import FormFieldCards
from .form_field_choice import FormFieldChoice
from .form_field_custom import FormFieldCustom
from .form_field_date import FormFieldDate
from .form_field_dropdown import FormFieldDropdown
from .form_field_email import FormFieldEmail
from .form_field_file import FormFieldFile
from .form_field_legal import FormFieldLegal
from .form_field_number import FormFieldNumber
from .form_field_password import FormFieldPassword
from .form_field_payment import FormFieldPayment
from .form_field_social import FormFieldSocial
from .form_field_tel import FormFieldTel
from .form_field_text import FormFieldText
from .form_field_url import FormFieldUrl

FormField = typing.Union[
    FormFieldBoolean,
    FormFieldCards,
    FormFieldChoice,
    FormFieldCustom,
    FormFieldDate,
    FormFieldDropdown,
    FormFieldEmail,
    FormFieldFile,
    FormFieldLegal,
    FormFieldNumber,
    FormFieldPassword,
    FormFieldPayment,
    FormFieldSocial,
    FormFieldTel,
    FormFieldText,
    FormFieldUrl,
]



from .conftest import get_client, verify_request_count


def test_actions_modules_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "actions.modules.list_.0"
    client = get_client(test_id)
    client.actions.modules.list(
        page=1,
        per_page=1,
    )
    verify_request_count(test_id, "GET", "/actions/modules", {"page": "1", "per_page": "1"}, 1)


def test_actions_modules_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "actions.modules.create.0"
    client = get_client(test_id)
    client.actions.modules.create(
        name="name",
        code="code",
    )
    verify_request_count(test_id, "POST", "/actions/modules", None, 1)


def test_actions_modules_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "actions.modules.get.0"
    client = get_client(test_id)
    client.actions.modules.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/actions/modules/id", None, 1)


def test_actions_modules_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "actions.modules.delete.0"
    client = get_client(test_id)
    client.actions.modules.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/actions/modules/id", None, 1)


def test_actions_modules_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "actions.modules.update.0"
    client = get_client(test_id)
    client.actions.modules.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/actions/modules/id", None, 1)


def test_actions_modules_list_actions() -> None:
    """Test listActions endpoint with WireMock"""
    test_id = "actions.modules.list_actions.0"
    client = get_client(test_id)
    client.actions.modules.list_actions(
        id="id",
        page=1,
        per_page=1,
    )
    verify_request_count(test_id, "GET", "/actions/modules/id/actions", {"page": "1", "per_page": "1"}, 1)


def test_actions_modules_rollback() -> None:
    """Test rollback endpoint with WireMock"""
    test_id = "actions.modules.rollback.0"
    client = get_client(test_id)
    client.actions.modules.rollback(
        id="id",
        module_version_id="module_version_id",
    )
    verify_request_count(test_id, "POST", "/actions/modules/id/rollback", None, 1)
